import pygame
import numpy as np


def surface2numpy(surface: pygame.Surface) -> np.ndarray:
    array = np.zeros((surface.get_width(), surface.get_height(), 4), dtype=np.uint8)
    array[..., 0] = pygame.surfarray.pixels_red(surface)
    array[..., 1] = pygame.surfarray.pixels_green(surface)
    array[..., 2] = pygame.surfarray.pixels_blue(surface)
    array[..., 3] = pygame.surfarray.pixels_alpha(surface)
    return array.transpose((1, 0, 2))


def numpy2surface(array: np.ndarray) -> pygame.Surface:
    assert array.ndim == 3
    assert array.shape[2] == 4

    array = array.transpose((1, 0, 2))
    surface = pygame.Surface((array.shape[0], array.shape[1]), flags=pygame.SRCALPHA)

    red = pygame.surfarray.pixels_red(surface)
    red[:, :] = array[..., 0]

    green = pygame.surfarray.pixels_green(surface)
    green[:, :] = array[..., 1]

    blue = pygame.surfarray.pixels_blue(surface)
    blue[:, :] = array[..., 2]

    alpha = pygame.surfarray.pixels_alpha(surface)
    alpha[...] = array[..., 3]

    return surface