import numpy as np

GROUND_ARRAY_INDEX = 0
GROUND_GRASS = 1
GROUND_SWAMP = 2
GROUND_DIRT = 3
GROUND_SAND = 4
GROUND_DEAD_LAND = 5
GROUND_MAX_VALUE = 5


class Region:

    def __init__(self, width: int = 1, height: int = 1):

        self.__cells = np.zeros([width, height, 1], dtype=np.int8)
        self.__cells[:, :, GROUND_ARRAY_INDEX] = GROUND_GRASS

    @property
    def cells(self) -> np.ndarray:
        return self.__cells

    @cells.setter
    def cells(self, array: np.ndarray):
        assert array.ndim == 3
        assert array.shape[2] >= 1
        self.__cells = array

    @property
    def width(self) -> int:
        return self.__cells.shape[0]

    @property
    def height(self) -> int:
        return self.__cells.shape[1]

    def getGroundValue(self, x: int, y: int) -> int:
        assert 0 <= x < self.width
        assert 0 <= y < self.height
        return self.__cells[x, y, GROUND_ARRAY_INDEX]

    def setGroundValue(self, x: int, y: int, value: int):
        assert 0 <= x < self.width
        assert 0 <= y < self.height
        assert 0 <= value <= GROUND_MAX_VALUE
        self.__cells[x, y, GROUND_ARRAY_INDEX] = value

    def getGroundNeightbors(self, x: int, y: int) -> np.array:
        assert 0 <= x < self.width
        assert 0 <= y < self.height
        width = self.width
        height = self.height
        if 0 < x < width - 1 and 0 < y < height - 1:
            # Common case, inside the map
            neightbors = self.__cells[x - 1:x + 2, y - 1:y + 2, GROUND_ARRAY_INDEX].copy()
        else:
            # Deal with corners
            neightbors = np.empty([3, 3], dtype=np.int8)
            neightbors.fill(self.__cells[x, y, GROUND_ARRAY_INDEX])
            if y > 0:
                if x > 0:
                    neightbors[0, 0] = self.__cells[x - 1, y - 1, GROUND_ARRAY_INDEX]
                neightbors[1, 0] = self.__cells[x, y - 1, GROUND_ARRAY_INDEX]
                if x < width - 1:
                    neightbors[2, 0] = self.__cells[x + 1, y - 1, GROUND_ARRAY_INDEX]
            if x > 0:
                neightbors[0, 1] = self.__cells[x - 1, y, GROUND_ARRAY_INDEX]
            if x < width - 1:
                neightbors[2, 1] = self.__cells[x + 1, y, GROUND_ARRAY_INDEX]
            if y < height - 1:
                if x > 0:
                    neightbors[0, 2] = self.__cells[x - 1, y + 1, GROUND_ARRAY_INDEX]
                neightbors[1, 2] = self.__cells[x, y + 1, GROUND_ARRAY_INDEX]
                if x < width - 1:
                    neightbors[2, 2] = self.__cells[x + 1, y + 1, GROUND_ARRAY_INDEX]

        return neightbors
