from __future__ import annotations

import io
from typing import Union

from state import State
from .IO import IO


class MainIO(IO):
    
    @staticmethod
    def createV1(stream: Union[None, io.BytesIO] = None) -> MainIO:
        from .v1.StateSerializer import StateSerializer
        from .v1.RegionSerializer import RegionSerializer

        mainIO = MainIO(stream)
        mainIO.registerObjectSerializer("State", StateSerializer())
        mainIO.registerObjectSerializer("Region", RegionSerializer())

        return mainIO

    @staticmethod
    def save(fileName: str, state: State):
        mainIO = MainIO.createV1()
        mainIO.createStream()
        mainIO.writeFileHeader("level", 1)
        mainIO.writeObject(state)
        mainIO.saveFile(fileName)

    @staticmethod
    def load(fileName: str) -> State:
        mainIO = IO()
        mainIO.loadFile(fileName)
        version = mainIO.readFileHeader("level")
        if version == 1:
            mainIO = MainIO.createV1(mainIO.stream)
            state = mainIO.readObject()
        else:
            raise RuntimeError("Unsupported version {}".format(version))
        return state
